"use strict";(self.webpackChunkobservable_jupyter_widget=self.webpackChunkobservable_jupyter_widget||[]).push([[480],{480:function(e,t,r){var i=this&&this.__createBinding||(Object.create?function(e,t,r,i){void 0===i&&(i=r),Object.defineProperty(e,i,{enumerable:!0,get:function(){return t[r]}})}:function(e,t,r,i){void 0===i&&(i=r),e[i]=t[r]}),n=this&&this.__setModuleDefault||(Object.create?function(e,t){Object.defineProperty(e,"default",{enumerable:!0,value:t})}:function(e,t){e.default=t}),u=this&&this.__importStar||function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var r in e)"default"!==r&&Object.prototype.hasOwnProperty.call(e,r)&&i(t,e,r);return n(t,e),t};Object.defineProperty(t,"__esModule",{value:!0});const a=r(565),o=u(r(367)),s=r(657),c={id:"observable-jupyter-widget:plugin",requires:[a.IJupyterWidgetRegistry],activate:function(e,t){t.registerWidget({name:s.MODULE_NAME,version:s.MODULE_VERSION,exports:o})},autoStart:!0};t.default=c}}]);